@@ -29,6 +29,7 @@ from liualgotrader.strategies.base import Strategy, StrategyType
 
 shortable: Dict = {}
 symbol_data_error: Dict = {}
+rejects: Dict[str, List[str]] = {}
 
 
 async def end_time(reason: str):
@@ -67,7 +68,7 @@ async def liquidator(trading_api: tradeapi) -> None:
             config.market_close - dt
             if config.market_close > dt
             else timedelta(hours=24) + (config.market_close - dt)
-        ) - timedelta(minutes=5)
+        ) - timedelta(minutes=15)
         await asyncio.sleep(to_market_close.total_seconds())
 
     except asyncio.CancelledError:
@@ -153,7 +154,7 @@ async def liquidate(
     trading_api: tradeapi,
 ) -> None:
 
-    if symbol_position:
+    if symbol_position and symbol not in trading_data.open_orders:
         tlog(
             f"Trading over, trying to liquidate remaining position {symbol_position} in {symbol}"
         )
@@ -380,6 +381,7 @@ async def handle_data_queue_msg(
 ) -> bool:
     global shortable
     global symbol_data_error
+    global rejects
 
     symbol = data["symbol"]
     if symbol not in market_data.minute_history:
@@ -496,7 +498,7 @@ async def handle_data_queue_msg(
         market_data.volume_today[symbol] = data["totalvolume"]
 
         if data["EV"] == "A":
-            if (time_diff := datetime.now(tz=timezone("America/New_York")) - original_ts) > timedelta(seconds=8):  # type: ignore
+            if (time_diff := datetime.now(tz=timezone("America/New_York")) - original_ts) > timedelta(seconds=10):  # type: ignore
                 tlog(f"A$ {symbol} too out of sync w {time_diff}")
                 return False
             elif (
@@ -572,6 +574,9 @@ async def handle_data_queue_msg(
                 ):
                     continue
 
+                if s.name in rejects and symbol in rejects[s.name]:
+                    continue
+
                 try:
                     do, what = await s.run(
                         symbol,
@@ -583,16 +588,21 @@ async def handle_data_queue_msg(
                         portfolio_value=config.portfolio_value,
                     )
                 except Exception as e:
-                    tlog(
-                        f"[EXCEPTION] strategy {s.name} for symbol {symbol} -> {e}"
-                    )
-                    exc_info = sys.exc_info()
-                    lines = traceback.format_exception(*exc_info)
-                    for line in lines:
-                        tlog(f"{line}")
-                    del exc_info
+                    # exc_info = sys.exc_info()
+                    # lines = traceback.format_exception(*exc_info)
+                    # for line in lines:
+                    #    tlog(f"{line}")
+                    # del exc_info
 
-                    if symbol not in symbol_data_error:
+                    symbol_data_error[symbol] = (
+                        0
+                        if symbol not in symbol_data_error
+                        else symbol_data_error[symbol] + 1
+                    )
+                    tlog(
+                        f"[EXCEPTION] strategy {s.name} for symbol {symbol} -> {e} [{symbol_data_error[symbol]}]"
+                    )
+                    if symbol_data_error[symbol] < 5:
                         tlog(f"attempting reload of data for symbol {symbol}")
 
                         _df = data_api.polygon.historic_agg_v2(
@@ -608,9 +618,6 @@ async def handle_data_queue_msg(
                         tlog(
                             f"consumer task re-loaded {len(market_data.minute_history[symbol].index)} 1-min candles for {symbol}"
                         )
-
-                        symbol_data_error[symbol] = True
-
                     continue
 
                 if do:
@@ -649,7 +656,12 @@ async def handle_data_queue_msg(
                         tlog(
                             f"Exception APIError with {e} from {what}, checking if order filled"
                         )
-
+                else:
+                    if what.get("reject", False):
+                        if s.name not in rejects:
+                            rejects[s.name] = [symbol]
+                        else:
+                            rejects[s.name].append(symbol)
     return True
 
 
@@ -786,8 +798,10 @@ async def consumer_async_main(
     strategy_types = []
     for strategy_name in strategies_conf:
         strategy_details = strategies_conf[strategy_name]
-        tlog(f"custom strategy {strategy_name} selected")
+        tlog(f"strategy {strategy_name} selected")
 
+        if strategy_details.get("off_hours", False):
+            tlog(f"{strategy_name} if off-hours, skipping during market hours")
         try:
             spec = importlib.util.spec_from_file_location(
                 "module.name", strategy_details["filename"]
@@ -799,9 +813,7 @@ async def consumer_async_main(
             custom_strategy = getattr(custom_strategy_module, class_name)
 
             if not issubclass(custom_strategy, Strategy):
-                tlog(
-                    f"custom strartegy must inherit from class {Strategy.__name__}"
-                )
+                tlog(f"strategy must inherit from class {Strategy.__name__}")
                 exit(0)
             strategy_details.pop("filename", None)
             strategy_types += [(custom_strategy, strategy_details)]
