@@ -9,7 +9,7 @@ import sys
 import traceback
 from datetime import datetime, timedelta
 from multiprocessing import Queue
-from queue import Empty
+from queue import Empty, Full
 from typing import Dict, List
 
 import alpaca_trade_api as tradeapi
@@ -159,7 +159,15 @@ async def run(
             elif (event_symbol := data.__dict__["_raw"]["symbol"]) in queue_id_hash:  # type: ignore
                 data.__dict__["_raw"]["EV"] = "T"
                 queue_id = queue_id_hash[event_symbol]
-                queues[queue_id].put(json.dumps(data.__dict__["_raw"]))
+                queues[queue_id].put(
+                    json.dumps(data.__dict__["_raw"]), timeout=1
+                )
+        except Full as f:
+            tlog(
+                f"[EXCEPTION] handle_trade_event():queue {queue_id} is FULL:{f}, sleep for 2 seconds and re-try."
+            )
+            await asyncio.sleep(2)
+            await handle_trade_event(conn, channel, data)
         except Exception as e:
             tlog(
                 f"Exception in handle_trade_event(): exception of type {type(e).__name__} with args {e.args}"
@@ -179,7 +187,16 @@ async def run(
             elif (event_symbol := data.__dict__["_raw"]["symbol"]) in queue_id_hash:  # type: ignore
                 data.__dict__["_raw"]["EV"] = "Q"
                 queue_id = queue_id_hash[event_symbol]
-                queues[queue_id].put(json.dumps(data.__dict__["_raw"]))
+                queues[queue_id].put(
+                    json.dumps(data.__dict__["_raw"]), timeout=1
+                )
+
+        except Full as f:
+            tlog(
+                f"[EXCEPTION] handle_quote_event():queue {queue_id} is FULL:{f}, sleeping for 2 seconds and re-trying."
+            )
+            await asyncio.sleep(2)
+            await handle_quote_event(conn, channel, data)
 
         except Exception as e:
             tlog(
@@ -209,8 +226,15 @@ async def run(
                         event_symbol
                     ]
                 queue_id = queue_id_hash[event_symbol]
-                queues[queue_id].put(json.dumps(data.__dict__["_raw"]))
-
+                queues[queue_id].put(
+                    json.dumps(data.__dict__["_raw"]), timeout=1
+                )
+        except Full as f:
+            tlog(
+                f"[EXCEPTION] handle_second_bar(): queue {queue_id} is FULL:{f}, sleeping for 2 seconds and re-trying."
+            )
+            await asyncio.sleep(2)
+            await handle_second_bar(conn, channel, data)
         except Exception as e:
             tlog(
                 f"Exception in handle_second_bar(): exception of type {type(e).__name__} with args {e.args}"
@@ -224,6 +248,7 @@ async def run(
         global symbol_strategy
         last_msg_tstamp = datetime.now()
 
+        queue_id: int = -1
         try:
             if (event_symbol := data.__dict__["_raw"]["symbol"]) in queue_id_hash:  # type: ignore
                 data.__dict__["_raw"]["EV"] = "AM"
@@ -234,10 +259,18 @@ async def run(
                     data.__dict__["_raw"]["symbol_strategy"] = symbol_strategy[
                         event_symbol
                     ]
-                queues[queue_id_hash[event_symbol]].put(
-                    json.dumps(data.__dict__["_raw"])
+                queue_id = queue_id_hash[event_symbol]
+                queues[queue_id].put(
+                    json.dumps(data.__dict__["_raw"]), timeout=1
                 )
 
+        except Full as f:
+            tlog(
+                f"[EXCEPTION] handle_minute_bar(): queue {queue_id} is FULL:{f}, sleeping for 2 seconds and re-trying."
+            )
+            await asyncio.sleep(2)
+            await handle_minute_bar(conn, channel, data)
+
         except Exception as e:
             tlog(
                 f"Exception in handle_minute_bar(): exception of type {type(e).__name__} with args {e.args}"
